from kraken.entrytypes.MetaData import MetaData
import copy
import traceback
import textwrap
import string

class Filter:
    def __init__(self):
        pass # placeholder for future
    
    def processMetaData(self, metadata):
       temp = MetaData()
       temp.keywords = copy.deepcopy(metadata.keywords)
       for a in metadata.entries:
          try: 
             if self.matches(a):
                temp.entries.append(copy.deepcopy(a))
          except:
             traceback.print_exc() 
             pass # exception in the evaluation means that it is not matching.
             # this might be a bit of problem the "Not filter", which will not
             # provide a true complement
       return temp
    
class NoFilter(Filter):
    def processMetaData(self, metadata):
        return copy.deepcopy(metadata)
        
    def matches(self, entry):
        return 1
        
    def __str__(self):
        return "(FILTER-PASS-ALL)"
    
class MultiFilter(Filter):
    def __init__(self):
        self.filters = []

    def __str_helper__(self, name):
        if self.filters == []:
            return "(" + name + " <EMPTY> )"
        tmp =  "(" + name + "\n"
        block = ""
        for flt in self.filters:
            block += str(flt) + ",\n"         
        vals = block[:-2].split("\n")    
        for a in vals:
          tmp += textwrap.fill(a, 80, initial_indent = " "*5, subsequent_indent = " "* 8) + "\n"
        # FIXME: use find or something to find the last ,
        tmp += ")"
        return tmp
        
        
class AndFilter(MultiFilter):    
    def __init__(self):
        MultiFilter.__init__(self)
        
    def matches(self, entry):
        for flt in self.filters:
            if not flt.matches(entry):
                return 0
        return 1
        
    def __str__(self):
        return self.__str_helper__("AND")

        
class OrFilter(MultiFilter):
    def __init__(self):
        MultiFilter.__init__(self)
        
    def matches(self, entry):
        for flt in self.filters:
            if flt.matches(entry):
                return 1
        return 0

    def __str__(self):
        return self.__str_helper__("OR")        
        

class NotFilter(Filter):
    def __init__(self):
        self.flt = None
        
    def matches(self, entry):
        return not self.flt.matches(entry)
        
    def __str__(self):
        return "(NOT " + str(self.flt) + ")"
                
'''
    This filter extracts a value from the entry.
'''        
class EvaluationFilter(Filter):
    def __init__(self):
        self.propertyName = None
        self.expression = None
        # the default fallback is false
        self.fallback = 0
        self.filterValue = None

    def getValue(self, entry):
        if self.propertyName != None:
            return entry.getExtendedProperty(self.propertyName)
            
        #if self.expression != None:
        #    return eval expression
        raise "Property name is None!!!"
        
class EqualityFilter(EvaluationFilter):
    def matches(self, entry):
        if self.getValue(entry) == self.filterValue:
            return 1
        return 0
        
    def __str__(self):
        return "(EQUALS " + self.propertyName + " " + self.filterValue + ")"

class SubstringFilter(EvaluationFilter):
    
    def __init__(self):
        self.ignoreCase = 1
    
    def matches(self, entry):
        value = self.getValue(entry)
        if value == None: 
            return 0
        if self.ignoreCase:
           if string.find(string.lower(value), string.lower(self.filterValue)) != -1:
               return 1
        else:    
           if string.find(value , self.filterValue) != -1:
               return 1
        return 0
        
    def __str__(self):
        return "(CONTAINS-SUBSTRING"  + self.filterValue + " " + self.propertyName + ")"

        
class FreeformatFilter(EvaluationFilter):    
    def matches(self, entry):
        for a in entry.listExtendedPropertyNames():
            self.propertyName = a
            value = string.lower(str(self.getValue(entry)))
            if string.find(string.lower(value), string.lower(self.filterValue)) != -1:
               return 1
        return 0        

    def __str__(self):
        return "(FREEFORMAT-CONTAINS " + self.filterValue + ")"
        

class RegexFilter(EvaluationFilter):
    def __init__(self):
        regobject = None
        EvaluationFilter.__init__(self)
    
    def matches(self, entry):
        if regobject == None:
            regobject = re.compile(self.filterValue)
        match = regobject.search(self.getValue(entry))            
        if  match != None:
            return 1
        return 0
        
    def __str__(self):
        return self.propertyName + " matches regex " + self.filterValue

        
class SmallerFilter(EvaluationFilter):
    def matches(self, entry):
        if self.getValue(entry) < self.filterValue:
            return 1
        return 0

    def __str__(self):
        return "(SMALLER-THAN " + self.propertyName + " " + self.filterValue + ")"
        
        
        
class GreaterFilter(EvaluationFilter):
    def matches(self, entry):
        if self.getValue(entry) > self.filterValue:
            return 1
        return 0

    def __str__(self):
        return self.propertyName + " greater than " + self.filterValue
        
class GreaterOrEqualFilter(EvaluationFilter):
    def matches(self, entry):
        if self.getValue(entry) <= self.filterValue:
            return 1
        return 0

    def __str__(self):
        return self.propertyName + " greater or equal with " + self.filterValue
        
'''
   Matches if any of the terms in the filterValue is in the sequence
   Filtervalue must be a sequence
'''        
class ContainsAnyFilter(EvaluationFilter):
    def matches(self, entry):
        val = self.getValue(entry)
        for a in self.filterValue:
            if a in val:
                return 1
        return 0
        
    def __str__(self):
        return "(CONTAINS-ANY-OF " + self.propertyName + " " + str(self.filterValue) + ")"
            
'''
   Matches if all of the terms in the filterValue is in the sequence
   Filtervalue must be a sequence
'''        
class ContainsAllFilter(EvaluationFilter):
    def matches(self, entry):
        val = self.getValue(entry)
        #print val
        #print self.filterValue
        for a in self.filterValue:
            if not (a in val):
                return 0
        return 1

    def __str__(self):
        return "(CONTAINS-ALL-OF " + self.propertyName + " " + str(self.filterValue) + ")"

        
'''
   Matches if the filterValue and the specified sentence are the same 
   except for order
'''        
class SameSetFilter(EvaluationFilter):    
    def matches(self, entry):
        val = self.getValue(entry)
        if len(val) != len(self.filterValue):
            return 0
        for a in self.filterValue:
            if not (a in val):
                return 0
        return 1
        